package org.jboss.security;

package .tmp;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.UndeclaredThrowableException;
import java.security.AccessControlContext;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import javax.security.auth.Subject;
import org.jboss.security.AbstractSecurityProxy;
import org.jboss.security.SubjectSecurityManager;

public class SubjectSecurityProxy extends AbstractSecurityProxy {
  private SubjectSecurityManager subjectSecurityManager;
  
  SubjectSecurityProxy(Object delegate) {
    super(delegate);
  }
  
  public void init(Class beanHome, Class beanRemote, Object securityMgr) throws InstantiationException {
    init(beanHome, beanRemote, null, null, securityMgr);
  }
  
  public void init(Class beanHome, Class beanRemote, Class beanLocalHome, Class beanLocal, Object securityMgr) throws InstantiationException {
    if (!(securityMgr instanceof SubjectSecurityManager)) {
      String msg = "SubjectSecurityProxy requires a SubjectSecurityManager instance, securityMgr=" + securityMgr;
      throw new InstantiationException(msg);
    } 
    this.subjectSecurityManager = (SubjectSecurityManager)securityMgr;
    super.init(beanHome, beanRemote, beanLocalHome, beanLocal, securityMgr);
  }
  
  protected void invokeHomeOnDelegate(Method m, Object[] args, Object delegate) throws SecurityException {
    Subject subject = this.subjectSecurityManager.getActiveSubject();
    if (subject == null)
      throw new SecurityException("No subject associated with secure proxy"); 
    try {
      Subject.doAsPrivileged(subject, (PrivilegedExceptionAction)new Object(this, m, delegate, args), (AccessControlContext)null);
    } catch (PrivilegedActionException e) {
      Throwable t = e.getException();
      if (t instanceof InvocationTargetException) {
        t = ((InvocationTargetException)t).getTargetException();
      } else if (t instanceof UndeclaredThrowableException) {
        t = ((UndeclaredThrowableException)t).getUndeclaredThrowable();
      } 
      if (t instanceof SecurityException)
        throw (SecurityException)t; 
      t.printStackTrace();
      throw new SecurityException("Unexpected error during security proxy execution:" + t.getMessage());
    } 
  }
  
  protected void invokeOnDelegate(Method m, Object[] args, Object delegate) throws Exception {
    Subject subject = this.subjectSecurityManager.getActiveSubject();
    if (subject == null)
      throw new SecurityException("No subject associated with secure proxy"); 
    try {
      Subject.doAsPrivileged(subject, (PrivilegedExceptionAction)new Object(this, m, delegate, args), (AccessControlContext)null);
    } catch (PrivilegedActionException e) {
      Throwable t = e.getException();
      if (t instanceof InvocationTargetException) {
        InvocationTargetException ex = (InvocationTargetException)t;
        t = ex.getTargetException();
        throw (Exception)t;
      } 
      if (t instanceof UndeclaredThrowableException)
        t = ((UndeclaredThrowableException)t).getUndeclaredThrowable(); 
      if (t instanceof SecurityException)
        throw (SecurityException)t; 
      throw new SecurityException("Unexpected error during security proxy execution:" + t.getMessage());
    } 
  }
}
